import numpy as np
import random
import datetime

MILLISECONDS_IN_SECOND = 1000.0
B_IN_MB = 1000000.0
BITS_IN_BYTE = 8.0
RANDOM_SEED = 42

VIDEO_CHUNCK_LEN = 2000.0  # millisec, every time add this amount to buffer
BITRATE_LEVELS = 4
BUFFER_LEVELS = 2
CHUNK_TIME_LEN = 2
Target_buffer = [2.0, 3.0]

lamda = 1
default_quality = 0
latency_threshold = 7
skip_add_frame = 100
ADD_FRAME = 0 

FPS = 25.0

class Environment:
    def __init__(self, all_cooked_time, all_cooked_bw, random_seed=RANDOM_SEED, logfile_path='./', VIDEO_SIZE_FILE ='./video_size_', Debug = True):
        assert len(all_cooked_time) == len(all_cooked_bw)
         
        if Debug:
            current_time = datetime.datetime.now().strftime('%Y-%m-%d_%H-%M-%S') 
            self.log_file = open(logfile_path +"log." +current_time, "w")

        self.video_size_file = VIDEO_SIZE_FILE
        self.Debug = Debug

        #np.random.seed(random_seed)

        self.all_cooked_time = all_cooked_time
        self.all_cooked_bw = all_cooked_bw

        self.time = 0
        self.play_time = 0
        self.play_time_counter = 0
        self.newest_frame = 0

        self.video_chunk_counter = 0
        self.buffer_size = 0

        # pick a random trace file
        self.trace_idx = 0
        self.cooked_time = self.all_cooked_time[self.trace_idx]
        self.cooked_bw = self.all_cooked_bw[self.trace_idx]
        # randomize the start point of the trace
        # note: trace file starts with time 0
        #self.mahimahi_ptr = np.random.randint(1, len(self.cooked_bw))
        #self.mahimahi_ptr = 1
        self.decision = False
        self.buffer_status = True
        self.skip_flag = False
        self.skip_time_frame = 100000000
        
        #self.last_mahimahi_time = self.cooked_time[self.mahimahi_ptr - 1]
        self.video_size = {}  # in bytes
        self.cdn_arrive_time = {}
        self.gop_time_len = {}
        self.gop_flag = {}

        for bitrate in range(BITRATE_LEVELS):
            self.video_size[bitrate] = []
            self.cdn_arrive_time[bitrate] = []
            self.gop_time_len[bitrate] = []
            self.gop_flag[bitrate] = []
            cnt = 0
            with open(self.video_size_file + str(bitrate)) as f:
                for line in f:
                    #print(line.split(), bitrate)
                    self.video_size[bitrate].append(float(line.split()[1]))
                    self.gop_time_len[bitrate].append(float(1/FPS))
                    self.gop_flag[bitrate].append(int(float(line.split()[2])))
                    self.cdn_arrive_time[bitrate].append(float(line.split()[0]))
                    cnt += 1
        self.gop_remain = self.video_size[default_quality][0]
        self.latency = self.gop_time_len[0][0] 
    def get_trace_id(self):
        return self.trace_idx

    def get_video_frame(self, quality, target_buffer):

        assert quality >= 0
        assert quality < BITRATE_LEVELS

        # Initial the Settings
        self.decision = False                                                 # GOP_flag
        video_frame_size = self.video_size[quality][self.video_chunk_counter] # Data_size
        cdn_rebuf_time = 0                                                    # CDN_rebuf_time
        rebuf = 0                                                             # rebuf_time
        FRAME_TIME_LEN = float(1/FPS)                                         # frame time len
        end_of_video = False                                                  # is trace time end
        duration = 0                                                          # this loop 's time len
        current_new = 0
        global ADD_FRAME
 
        if target_buffer == 0:
            quick_play_bound = 3
            slow_play_bound = 1.0
        else:
            quick_play_bound = 4
            slow_play_bound = 1.5
        # This code is check the quick play or slow play
        # output is the play_weight
        if self.buffer_size > quick_play_bound :
            #quick play
            play_duration_weight = 1.05
            #if Debug:
                #print("kuaibo\n") 
            #elif self.buffer_size < slow_play_bound:
        elif  self.buffer_size < slow_play_bound :
            #slow play
            play_duration_weight = 0.95
            #if Debug:
                #print("manbo\n")
        else:
            play_duration_weight = 1
        
        # This code is check Is the cdn has the frame in this time
        # self.time means the real time
        # self.cdn_arrive_time means the time the frame came
        if self.time < self.cdn_arrive_time[quality][self.video_chunk_counter] and not end_of_video: # CDN can't get the frame
            cdn_rebuf_time = self.cdn_arrive_time[quality][self.video_chunk_counter] - self.time
            self.newest_frame = self.video_chunk_counter
            duration = cdn_rebuf_time
            # if the client means the buffering
            if not self.buffer_status:
                # not buffering ,you can't get the frame ,but you must play the frame

                # the buffer is enough
                if self.buffer_size > cdn_rebuf_time * play_duration_weight:
                    self.buffer_size -= cdn_rebuf_time * play_duration_weight
                    self.play_time += cdn_rebuf_time * play_duration_weight
                    rebuf = 0
                    play_len = cdn_rebuf_time * play_duration_weight 
                # not enough .let the client buffering
                else:
                    self.play_time += self.buffer_size
                    rebuf = cdn_rebuf_time  - (self.buffer_size / play_duration_weight)
                    play_len = self.buffer_size 
                    self.buffer_size = 0
                    self.buffer_status = True
               
            # calculate the play time , the real time ,the latency
                # the normal play the frame
                if self.skip_flag and self.play_time_counter >= self.skip_time_frame :
                    self.play_time_counter += ADD_FRAME
                    self.play_time = self.play_time_counter * FRAME_TIME_LEN
                    self.skip_flag = False
                    if self.Debug:
                        self.log_file.write("ADD_Frame" + str(ADD_FRAME) + "\n")
                    #print(ADD_FRAME)
                    ADD_FRAME = 0
                else:
                    self.play_time_counter = int(self.play_time/FRAME_TIME_LEN)

                self.latency = (self.newest_frame - self.video_chunk_counter) * FRAME_TIME_LEN  + self.buffer_size
                self.time = self.cdn_arrive_time[quality][self.video_chunk_counter]
            else:
                rebuf = duration
                play_len = 0
                self.time = self.cdn_arrive_time[quality][self.video_chunk_counter]
                self.latency = (self.newest_frame - self.video_chunk_counter) * FRAME_TIME_LEN  + self.buffer_size
            # Debug info 
            '''print("%.4f"%self.time ,
                  "  cdn %4f"%cdn_rebuf_time, 
                  "~rebuf~~ %.3f"%rebuf,
                  "~dur~~%.4f"%duration,
                  "~delay~%.4f"%(cdn_rebuf_time),
                  "~id! ", self.video_chunk_counter,
                  "~newest ", self.newest_frame,
                  "~~buffer~~ %4f"%self.buffer_size, 
                  "~~play_time~~%4f"%self.play_time ,
                  "~~play_id",self.play_time_counter,
                  "~~latency~~%4f"%self.latency,"000")'''
            if self.Debug:
                self.log_file.write("real_time %.4f\t"%self.time +  
                                  "cdn_rebuf%.4f\t"%cdn_rebuf_time + 
                                  "client_rebuf %.3f\t"%rebuf  + 
                                  "download_duration %.4f\t"%duration + 
                                  "frame_size %.4f\t"%video_frame_size + 
                                  "play_time_len %.4f\t"% (play_len) +
                                  "download_id %d\t"%(self.video_chunk_counter-1) + 
                                  "cdn_newest_frame %d\t"%self.newest_frame + 
                                  "client_buffer %.4f\t"%self.buffer_size  +
                                  "play_time %.4f\t"%self.play_time + 
                                  "play_id %.4f\t"%self.play_time_counter + 
                                  "latency %.4f\t"%self.latency + "000\n")
            # Return the loop
            cdn_has_frame = []
            for bitrate in range(BITRATE_LEVELS):
                cdn_has_frame_temp = self.video_size[bitrate][self.video_chunk_counter : self.newest_frame]
                cdn_has_frame.append(cdn_has_frame_temp)
            cdn_has_frame.append(self.gop_flag[bitrate][self.video_chunk_counter:self.newest_frame])

            return  [self.time,                             # physical time
                    duration,                               # this loop duration, means the download time
                    0,                                      # frame data size
                    0,                                      # frame time len
                    rebuf,                                  # rebuf len
                    self.buffer_size,                       # client buffer
                    play_len,                               # play time len
                    self.latency ,                          # latency
                    self.newest_frame,                      # cdn the newest frame id
                    (self.video_chunk_counter - 1),         # download_id 
                    cdn_has_frame,                          # CDN_has_frame
                    self.decision,                          # Is_I frame edge
                    self.buffer_status,                     # Is the buffer is buffering
                    True,                                   # Is the CDN has no frame
                    end_of_video]                          # Is the end of video
        else:
            the_newst_frame = self.video_chunk_counter
            current_new = self.cdn_arrive_time[quality][the_newst_frame]
            while( current_new < self.time):
                the_newst_frame += 1
                current_new = self.cdn_arrive_time[quality][the_newst_frame]
            self.newest_frame = the_newst_frame
        # If the CDN can get the frame:
        if int(self.time / 0.5) >= len(self.cooked_bw):
            end_of_video = True
        else:
            throughput = self.cooked_bw[int(self.time / 0.5)]  * B_IN_MB
            #rtt        = self.cooked_rtt[int(self.time / 0.5)]
            duration = float(video_frame_size / throughput)
        # If the the frame is the Gop end ,next will be the next I frame
        if self.gop_flag[quality][self.video_chunk_counter + 1] == 1:
            self.decision = True 
        # If the buffering
        if self.buffer_status and not end_of_video:
            # let the buffer_size to expand to the target_buffer
            if self.buffer_size < Target_buffer[target_buffer]:
                rebuf = duration
                self.buffer_size += self.gop_time_len[quality][self.video_chunk_counter]
                self.time += duration
            # if the buffer is enough
            else:
                self.buffer_status = False
                rebuf = duration
            # calculate the play time , the real time ,the latency
            self.play_time_counter = int(self.play_time/FRAME_TIME_LEN) 
            self.latency = (self.newest_frame - self.video_chunk_counter) * FRAME_TIME_LEN + self.buffer_size
            # Debug Info
            if self.latency > latency_threshold and not self.skip_flag :
                self.skip_flag = True
                self.skip_time_frame = self.video_chunk_counter
                if self.newest_frame >  skip_add_frame + self.video_chunk_counter:
                    #ADD_FRAME = self.play_time_counter + skip_add_frame - self.video_chunk_counter 
                    ADD_FRAME =  skip_add_frame
                    self.video_chunk_counter = self.video_chunk_counter + skip_add_frame 
                else:
                    ADD_FRAME =  self.newest_frame - self.video_chunk_counter
                    self.video_chunk_counter = self.newest_frame
                rebuf += ADD_FRAME * FRAME_TIME_LEN  
                if self.Debug:
                    self.log_file.write("skip events: skip_download_frame, play_frame, new_download_frame, ADD_frame" + str(self.skip_time_frame) + " " + str(self.play_time_counter) +" " + str(self.video_chunk_counter) +" " +str(ADD_FRAME) + "\n")
            else:
                self.video_chunk_counter += 1
            '''print("%.4f"%self.time ,
                      "  cdn %4f"%cdn_rebuf_time, 
                      "~rebuf~~ %.3f"%rebuf,
                      "~dur~~%.4f"%duration,
                      "~delay~%.4f"%(cdn_rebuf_time),
                      "~id! ", self.video_chunk_counter,
                      "~newest ", self.newest_frame,
                      "~~buffer~~ %4f"%self.buffer_size, 
                      "~~play_time~~%4f"%self.play_time ,
                      "~~play_id",self.play_time_counter,
                      "~~latency~~%4f"%self.latency,"111")'''
            if self.Debug:
                 self.log_file.write("real_time %.4f\t"%self.time +  
                                      "cdn_rebuf%.4f\t"%cdn_rebuf_time + 
                                      "client_rebuf %.3f\t"%rebuf  + 
                                      "download_duration %.4f\t"%duration + 
                                      "frame_size %.4f\t"%video_frame_size + 
                                      "play_time len %.4f\t"% 0 +
                                      "download_id %d\t"%(self.video_chunk_counter-1) + 
                                      "cdn_newest_frame %d\t"%self.newest_frame + 
                                      "client_buffer %.4f\t"%self.buffer_size  +
                                      "play_time %.4f\t"%self.play_time + 
                                      "play_id %.4f\t"%self.play_time_counter + 
                                      "latency %.4f\t"%self.latency + "111\n")
            cdn_has_frame = []
            for bitrate in range(BITRATE_LEVELS):
                cdn_has_frame_temp = self.video_size[bitrate][self.video_chunk_counter : self.newest_frame]
                cdn_has_frame.append(cdn_has_frame_temp)
            cdn_has_frame.append(self.gop_flag[bitrate][self.video_chunk_counter:self.newest_frame])
            # Return the loop
            return      [self.time,                             # physical time
                        duration,                               # this loop duration, means the download time
                        video_frame_size,                       # frame data size
                        FRAME_TIME_LEN,                         # frame time len
                        rebuf,                                  # rebuf len
                        self.buffer_size,                       # client buffer
                        0,                                      # play time len
                        self.latency ,                          # latency
                        self.newest_frame,                      # cdn the newest frame id
                        (self.video_chunk_counter - 1),         # download_id
                        cdn_has_frame,                          # CDN_has_frame
                        self.decision,                          # Is_I frame edge
                        self.buffer_status,                     # Is the buffer is buffering
                        False,                                  # Is the CDN has no frame
                        end_of_video]                           # Is the end of video
        # If not buffering
        elif not end_of_video: 
            # the normal loop
            # the buffer is enough
            if self.buffer_size > duration * play_duration_weight:
                self.buffer_size -= duration * play_duration_weight
                self.play_time += duration * play_duration_weight 
                rebuf = 0
            # the buffer not enough
            else:
                self.play_time += self.buffer_size
                rebuf = duration  - (self.buffer_size / play_duration_weight)
                self.buffer_size = 0
                self.buffer_status = True
            # the normal play the frame
            if self.skip_flag and self.play_time_counter >= self.skip_time_frame :
                self.play_time_counter += ADD_FRAME
                self.play_time = self.play_time_counter * FRAME_TIME_LEN
                self.skip_flag = False
                if self.Debug:
                    self.log_file.write("ADD_Frame" + str(ADD_FRAME) + "\n")
                #print(ADD_FRAME)
                ADD_FRAME = 0
            else:
                self.play_time_counter = int(self.play_time/FRAME_TIME_LEN) 
            self.latency = (self.newest_frame - self.video_chunk_counter) * FRAME_TIME_LEN + self.buffer_size
            #play add the time
            self.buffer_size += self.gop_time_len[quality][self.video_chunk_counter]
            self.time += duration
            if self.latency > latency_threshold and not self.skip_flag:
                self.skip_flag = True
                self.skip_time_frame = self.video_chunk_counter
                if self.video_chunk_counter + skip_add_frame < self.newest_frame:
                    #ADD_FRAME = self.play_time_counter + skip_add_frame - self.video_chunk_counter
                    ADD_FRAME = skip_add_frame
                    self.video_chunk_counter = self.video_chunk_counter + skip_add_frame
                else:
                    ADD_FRAME = self.newest_frame - self.video_chunk_counter
                    self.video_chunk_counter = self.newest_frame
                rebuf += ADD_FRAME * FRAME_TIME_LEN  
                if self.Debug:
                    self.log_file.write("skip events: skip_download_frame, play_frame, new_download_frame, ADD_frame" + str(self.skip_time_frame) + " " + str(self.play_time_counter) +" " + str(self.video_chunk_counter) +" " +str(ADD_FRAME) + "\n")
            else:
                self.video_chunk_counter += 1
            '''print("%.4f"%self.time ,
                      "  cdn %4f"%cdn_rebuf_time, 
                      "~rebuf~~ %.3f"%rebuf,
                      "~dur~~%.4f"%duration,
                      "~delay~%.4f"%(cdn_rebuf_time),
                      "~id! ", self.video_chunk_counter,
                      "~newest ", self.newest_frame,
                      "~~buffer~~ %4f"%self.buffer_size, 
                      "~~play_time~~%4f"%self.play_time ,
                      "~~play_id",self.play_time_counter,
                      "~~latency~~%4f"%self.latency,"222")'''
            if self.Debug:
                 self.log_file.write("real_time %.4f\t"%self.time +  
                                      "cdn_rebuf%.4f\t"%cdn_rebuf_time + 
                                      "client_rebuf %.3f\t"%rebuf  + 
                                      "download_duration %.4f\t"%duration + 
                                      "frame_size %.4f\t"%video_frame_size + 
                                      "play_time_len %.4f\t"% (duration * play_duration_weight) +
                                      "download_id %d\t"%(self.video_chunk_counter-1) + 
                                      "cdn_newest_frame %d\t"%self.newest_frame + 
                                      "client_buffer %.4f\t"%self.buffer_size  +
                                      "play_time %.4f\t"%self.play_time + 
                                      "play_id %.4f\t"%self.play_time_counter + 
                                      "latency %.4f\t"%self.latency + "222\n")
            cdn_has_frame = []
            for bitrate in range(BITRATE_LEVELS):
                cdn_has_frame_temp = self.video_size[bitrate][self.video_chunk_counter : self.newest_frame]
                cdn_has_frame.append(cdn_has_frame_temp)
            cdn_has_frame.append(self.gop_flag[bitrate][self.video_chunk_counter:self.newest_frame])
            #return loop
            return      [self.time,                             # physical time
                        duration,                               # this loop duration, means the download time
                        video_frame_size,                       # frame data size
                        FRAME_TIME_LEN,                         # frame time len
                        rebuf,                                  # rebuf len
                        self.buffer_size,                       # client buffer
                        (duration * play_duration_weight),      # play time len
                        self.latency ,                          # latency
                        self.newest_frame,                      # cdn the newest frame id
                        (self.video_chunk_counter - 1),         # download_id
                        cdn_has_frame,                          # CDN_has_frame
                        self.decision,                          # Is_I frame edge
                        self.buffer_status,                     # Is the buffer is buffering
                        False,                                  # Is the CDN has no frame
                        end_of_video]                           # Is the end of video
        # if the video is end
        if  end_of_video:
            self.time = 0
            self.play_time = 0
            self.play_time_counter = 0
            self.newest_frame = 0

            self.video_chunk_counter = 0
            self.buffer_size = 0

            # pick a random trace file
            self.trace_idx += 1
            if self.trace_idx >= len(self.all_cooked_time):
                self.trace_idx = 0 
            #self.trace_idx += 1
            #if self.trace_idx >= len(self.all_cooked_time):
            #    self.trace_idx = 0 
            self.cooked_time = self.all_cooked_time[self.trace_idx]
            self.cooked_bw = self.all_cooked_bw[self.trace_idx]
       
            self.decision = False
            self.buffer_status = True
            self.skip_flag = False
            self.skip_time_frame = 100000000
            ADD_FRAME = 0

            self.video_size = {}  # in bytes
            self.cdn_arrive_time = {}
            self.gop_time_len = {}
            self.gop_flag = {}
            for bitrate in range(BITRATE_LEVELS):
                self.video_size[bitrate] = []
                self.cdn_arrive_time[bitrate] = []
                self.gop_time_len[bitrate] = []
                self.gop_flag[bitrate] = []
                cnt = 0
                with open(self.video_size_file + str(bitrate)) as f:
                    for line in f:
                        self.video_size[bitrate].append(float(line.split()[1]))
                        self.gop_time_len[bitrate].append(float(1/FPS))
                        self.gop_flag[bitrate].append(int(float(line.split()[2])))
                        self.cdn_arrive_time[bitrate].append(float(line.split()[0]))
                        cnt += 1
            self.gop_remain = self.video_size[default_quality][0]
            self.latency = self.gop_time_len[0][0] 
            cdn_has_frame = []
            for bitrate in range(BITRATE_LEVELS):
                cdn_has_frame_temp = self.video_size[bitrate][self.video_chunk_counter : self.newest_frame]
                cdn_has_frame.append(cdn_has_frame_temp)
            cdn_has_frame.append(self.gop_flag[bitrate][self.video_chunk_counter:self.newest_frame])
            return      [self.time,                             # physical time
                        duration,                               # this loop duration, means the download time
                        video_frame_size,                       # frame data size
                        FRAME_TIME_LEN,                         # frame time len
                        rebuf,                                  # rebuf len
                        self.buffer_size,                       # client buffer
                        (duration * play_duration_weight),      # play time len
                        self.latency ,                          # latency
                        self.newest_frame,                      # cdn the newest frame id
                        (self.video_chunk_counter - 1),         # download_id
                        cdn_has_frame,                          # CDN_has_frame
                        self.decision,                          # Is_I frame edge
                        self.buffer_status,                     # Is the buffer is buffering
                        False,                                  # Is the CDN has no frame
                        True]                                   # Is the end of video



class Algorithm:
     def __init__(self):
     # fill your init vars
         self.buffer_size = 0
         self.rebuf_time = [0] * 20
     # Intial 
     def Initial(self):
         self.buffer_size = 0

     #Define your al
     def run(self, S_time_interval, S_send_data_size, S_frame_time_len, S_frame_type, S_buffer_size, S_end_delay, rebuf_time, cdn_has_frame,cdn_flag, buffer_flag):
         self.rebuf_time.append(rebuf_time) 
         self.rebuf_time.pop(0)
         if S_buffer_size[-1] > 1.0:
             bit_rate = 1
         else:
             bit_rate = 0
         if sum(self.rebuf_time) < 0.001 and bit_rate == 0:
             bit_rate = 1
         target_buffer = 1.5
         return bit_rate, target_buffer





''' Demo SDK for LiveStreaming
    Author Dan Yang
    Time 2018-12-15
    For LiveStreaming final Game'''
# import the env from pip
#import LiveStreamingEnv.fixed_final_env as env
import final_fixed_env as env
import LiveStreamingEnv.load_trace as load_trace
#import matplotlib.pyplot as plt
import time
import numpy as np
import ABR
# path setting
TRAIN_TRACES = './network_trace/'   #train trace path setting,
#video_size_file = './video_trace/AsianCup_China_Uzbekistan/frame_trace_'      #video trace path setting,
video_size_file = './video_trace/Dingdian/frame_trace_'      #video trace path setting,
LogFile_Path = "./log/"                #log file trace path setting,
# Debug Mode: if True, You can see the debug info in the logfile
#             if False, no log ,but the training speed is high
DEBUG = False
DRAW = False
# load the trace
all_cooked_time, all_cooked_bw, all_file_names = load_trace.load_trace(TRAIN_TRACES)
#random_seed 
random_seed = 2
video_count = 0
#FPS = 25
frame_time_len = 0.04
reward_all_sum = 0
#init the environment
#setting one:
#     1,all_cooked_time : timestamp
#     2,all_cooked_bw   : throughput
#     3,all_cooked_rtt  : rtt
#     4,agent_id        : random_seed
#     5,logfile_path    : logfile_path
#     6,VIDEO_SIZE_FILE : Video Size File Path
#     7,Debug Setting   : Debug
net_env = env.Environment(all_cooked_time=all_cooked_time,
			  all_cooked_bw=all_cooked_bw,
			  random_seed=random_seed,
			  logfile_path=LogFile_Path,
			  VIDEO_SIZE_FILE=video_size_file,
			  Debug = DEBUG)
BIT_RATE      = [500.0,1200.0] # kpbs
TARGET_BUFFER = [2.0,3.0]   # seconds
# ABR setting
RESEVOIR = 0.5
CUSHION  = 2

cnt = 0
# defalut setting
last_bit_rate = 0
bit_rate = 0
target_buffer = 1.5
# QOE setting
reward_frame = 0
reward_all = 0
SMOOTH_PENALTY= 0.02 
REBUF_PENALTY = 1.5 
LANTENCY_PENALTY = 0.005 

# plot info
idx = 0
id_list = []
bit_rate_record = []
buffer_record = []
throughput_record = []
# plot the real time image
if DRAW:
    fig = plt.figure()
    plt.ion()
    plt.xlabel("time")
    plt.axis('off')
call_cnt = 0
call_time = 0
switch_num = 0

S_time_interval = []
S_send_data_size = []
S_frame_type = []
S_frame_time_len = []
S_buffer_size = []
S_end_delay = []
cdn_has_frame = []
rebuf_time = 0
buffer_flag = 0
cdn_flag=0

abr = ABR.Algorithm()
abr.Initial()

while True:
        reward_frame = 0
        # input the train steps
        #if cnt > 1200:
            #plt.ioff()
        #    break
        #actions bit_rate  target_buffer
        # every steps to call the environment
        # time           : physical time 
        # time_interval  : time duration in this step
        # send_data_size : download frame data size in this step
        # chunk_len      : frame time len
        # rebuf          : rebuf time in this step          
        # buffer_size    : current client buffer_size in this step          
        # rtt            : current buffer  in this step          
        # play_time_len  : played time len  in this step          
        # end_delay      : end to end latency which means the (upload end timestamp - play end timestamp)
        # decision_flag  : Only in decision_flag is True ,you can choose the new actions, other time can't Becasuse the Gop is consist by the I frame and P frame. Only in I frame you can skip your frame
        # buffer_flag    : If the True which means the video is rebuffing , client buffer is rebuffing, no play the video
        # cdn_flag       : If the True cdn has no frame to get 
        # end_of_video   : If the True ,which means the video is over.
        time, time_interval, send_data_size, frame_time_len, rebuf, buffer_size, end_delay, cdn_newest_id, downlaod_id, cdn_has_frame, decision_flag, buffer_flag,switch,cdn_flag, end_of_video = net_env.get_video_frame(bit_rate,target_buffer)
        cnt += 1
        call_time += time_interval
        switch_num += switch

        S_time_interval.append(time_interval)
        S_buffer_size.append(buffer_size)
        S_send_data_size.append(send_data_size)
        S_frame_time_len.append(frame_time_len)
        S_end_delay.append(end_delay)
        if decision_flag:
            S_frame_type.append(1)
        else:
            S_frame_type.append(0)
        rebuf_time += rebuf
        '''if time_interval != 0:
            # plot bit_rate 
            id_list.append(idx)
            idx += time_interval
            bit_rate_record.append(BIT_RATE[bit_rate])
            # plot buffer 
            buffer_record.append(buffer_size)
            # plot throughput 
            trace_idx = net_env.get_trace_id()
            print(trace_idx, idx,len(all_cooked_bw[trace_idx]))
            throughput_record.append(all_cooked_bw[trace_idx][int(idx/0.5)] * 1000 )'''
        if not cdn_flag:
            reward_frame = frame_time_len * float(BIT_RATE[bit_rate]) / 1000  - REBUF_PENALTY * rebuf - LANTENCY_PENALTY * end_delay
        else:
            reward_frame = -(REBUF_PENALTY * rebuf)
        
        #if decision_flag or end_of_video:
            # reward formate = play_time * BIT_RATE - 4.3 * rebuf - 1.2 * end_delay
        #    reward_frame += -1 * SMOOTH_PENALTY * (abs(BIT_RATE[bit_rate] - BIT_RATE[last_bit_rate]) / 1000)
            # last_bit_rate
        #    last_bit_rate = bit_rate
        if call_time > 0.5 and not end_of_video:
            reward_frame += -(switch_num) * SMOOTH_PENALTY * (1200 - 500) / 1000
            
            bit_rate , target_buffer = abr.run(S_time_interval, S_send_data_size,S_frame_time_len,S_frame_type,S_buffer_size,S_end_delay,\
                                      rebuf_time, cdn_has_frame, cdn_flag, buffer_flag)

            call_time = 0
            switch_num = 0
            call_cnt += 1

            S_time_interval= []
            S_send_data_size = []
            S_frame_type = []
            S_frame_time_len = []
            S_buffer_size = []
            S_end_delay = []
            rebuf_time = 0

        # --`----------------------------------------- End  ------------------------------------------- 

        reward_all += reward_frame
        if end_of_video:
            # Narrow the range of results
            print("video count", video_count, reward_all)
            reward_all_sum += reward_all / 100
            video_count += 1
            if video_count >= len(all_file_names):
                break
            cnt = 0
            reward_all = 0
            last_bit_rate = 0
            bit_rate = 0
            target_buffer = 1.5

            S_time_interval = []
            S_send_data_size = []
            S_frame_type = []
            S_frame_time_len = []
            S_buffer_size = []
            S_end_delay = []
            cdn_has_frame = []
            rebuf_time = 0
            buffer_flag = 0
            cdn_flag=0

            call_cnt = 0
            call_time = 0
            switch_num = 0
if DRAW:
    plt.show()
print(reward_all_sum)




import os


COOKED_TRACE_FOLDER = './cooked_traces/'


def load_trace(cooked_trace_folder=COOKED_TRACE_FOLDER):
    cooked_files = os.listdir(cooked_trace_folder)
    all_cooked_time = []
    all_cooked_bw = []
    all_file_names = []
    for cooked_file in cooked_files:
        file_path = cooked_trace_folder + cooked_file
        cooked_time = []
        cooked_bw = []
        # print file_path
        with open(file_path, 'rb') as f:
            for line in f:
                parse = line.split()
                cooked_time.append(float(parse[0]))
                cooked_bw.append(float(parse[1]))
        all_cooked_time.append(cooked_time)
        all_cooked_bw.append(cooked_bw)
        all_file_names.append(cooked_file)

    return all_cooked_time, all_cooked_bw, all_file_names



def get_params():
    params = []
    # your indicator
    video_type = 1
    #------- fill your params ------

    # ------------------------------ 
    params.append(video_type)
    return params 

